using Serilog;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using System.Collections.Generic;
using System;
using System.Threading.Tasks;

// Comandos que procesan las DTOs.
namespace EvaPosSrvCmd
{
    /// <summary>
    /// Interface de definición de comandos.
    /// </summary>
    public interface IComando
    {
        /// <summary>
        /// Identifica el mensaje al cual el comando responde.
        /// La referencia es usada por el servidor de mensajes para asociar 
        /// un tipo de mensaje con el comando respectivo.
        /// La referencia corresponde a un string contenido en el mensaje
        /// que permite identificar su tipo.
        /// </summary>
        public string Referencia { get; set; }

        /// <summary>
        /// Carga el DTO con los datos a procesar.
        /// Este método debe ser invocado antes de ejecutar el comando,
        /// para inicializarlo con los datos necesarios para ejecutar.
        /// </summary>
        public void CargaDTO(DTOBase dto);

        /// <summary>
        /// Ejecuta el comando, retorna objeto DTO de respuesta.
        /// </summary>
        public Respuestas Ejecutar();

        /// <summary>
        /// Retorna una "shallow copy" del objeto.
        /// </summary>
        public IComando CreaCopia();
    }

    /// <summary>
    /// Directorio de comandos.
    /// Almacena instancia de comandos indexadas por su Referencia.
    /// Usado para identificar el comando que procesa un mensaje particular
    /// asociado por la propiedad Referencia del comando.
    /// </summary>
    public class DirectorioCmds
    {
        private Dictionary<string, IComando> dir = new Dictionary<string, IComando>();
        
        /// <summary>
        /// Agrega un comando al directorio.
        /// </summary>
        public void AgregaComando(IComando cmd)
        {
            dir.Add(cmd.Referencia, cmd);
        }

        /// <summary>
        /// Recupera una "shallow copy" del comando asociado al parámmetro.
        /// </summary>
        public IComando ObtieneComando(string referencia)
        {
            IComando cmd;
            try {
                cmd = dir[referencia];
            } catch (KeyNotFoundException) 
            {
                throw new Exception($"No se encuentra comando con referencia '{referencia}'.");
            }
            return cmd.CreaCopia();
        }
    }
}